diff mbox

mm: thp: register mm for khugepaged when merging vma for shmem

Message ID 1529617247-126312-1-git-send-email-yang.shi@linux.alibaba.com (mailing list archive)
State New, archived
Headers show

Commit Message

Yang Shi June 21, 2018, 9:40 p.m. UTC
When merging anonymous page vma, if the size of vam can fit in at least
one hugepage, the mm will be registered for khugepaged for collapsing
THP in the future.

But, it skips shmem vma. Doing so for shmem too when merging vma in
order to increase the odd to collapse hugepage by khugepaged.

Also increase the count of shmem THP collapse. It looks missed before.

Signed-off-by: Yang Shi <yang.shi@linux.alibaba.com>
Cc: Hugh Dickins <hughd@google.com>
Cc: Kirill A. Shutemov <kirill.shutemov@linux.intel.com>
Cc: Vlastimil Babka <vbabka@suse.cz>
---
 mm/khugepaged.c | 10 ++++++++--
 1 file changed, 8 insertions(+), 2 deletions(-)

Comments

Kirill A . Shutemov June 21, 2018, 10:10 p.m. UTC | #1
On Fri, Jun 22, 2018 at 05:40:47AM +0800, Yang Shi wrote:
> When merging anonymous page vma, if the size of vam can fit in at least

s/vam/vma/

> one hugepage, the mm will be registered for khugepaged for collapsing
> THP in the future.
> 
> But, it skips shmem vma. Doing so for shmem too when merging vma in
> order to increase the odd to collapse hugepage by khugepaged.

Good catch. Thanks.

I think the fix incomplete. We shouldn't require vma->anon_vma for shmem,
only for anonymous mappings. We don't support file-private THPs.

> Also increase the count of shmem THP collapse. It looks missed before.

Separate patch, please.
Yang Shi June 21, 2018, 10:24 p.m. UTC | #2
On 6/21/18 3:10 PM, Kirill A. Shutemov wrote:
> On Fri, Jun 22, 2018 at 05:40:47AM +0800, Yang Shi wrote:
>> When merging anonymous page vma, if the size of vam can fit in at least
> s/vam/vma/
>
>> one hugepage, the mm will be registered for khugepaged for collapsing
>> THP in the future.
>>
>> But, it skips shmem vma. Doing so for shmem too when merging vma in
>> order to increase the odd to collapse hugepage by khugepaged.
> Good catch. Thanks.
>
> I think the fix incomplete. We shouldn't require vma->anon_vma for shmem,
> only for anonymous mappings. We don't support file-private THPs.

So you mean, shmem_file(vma->vm_file) && PageAnon(page)?

>
>> Also increase the count of shmem THP collapse. It looks missed before.
> Separate patch, please.

Sure.

Thanks,
Yang

>
Yang Shi June 21, 2018, 10:49 p.m. UTC | #3
On 6/21/18 3:24 PM, Yang Shi wrote:
>
>
> On 6/21/18 3:10 PM, Kirill A. Shutemov wrote:
>> On Fri, Jun 22, 2018 at 05:40:47AM +0800, Yang Shi wrote:
>>> When merging anonymous page vma, if the size of vam can fit in at least
>> s/vam/vma/
>>
>>> one hugepage, the mm will be registered for khugepaged for collapsing
>>> THP in the future.
>>>
>>> But, it skips shmem vma. Doing so for shmem too when merging vma in
>>> order to increase the odd to collapse hugepage by khugepaged.
>> Good catch. Thanks.
>>
>> I think the fix incomplete. We shouldn't require vma->anon_vma for 
>> shmem,
>> only for anonymous mappings. We don't support file-private THPs.
>
> So you mean, shmem_file(vma->vm_file) && PageAnon(page)?

I got your point. Please disregard the question.

>
>>
>>> Also increase the count of shmem THP collapse. It looks missed before.
>> Separate patch, please.
>
> Sure.
>
> Thanks,
> Yang
>
>>
>
diff mbox

Patch

diff --git a/mm/khugepaged.c b/mm/khugepaged.c
index d7b2a4b..27f5ce2 100644
--- a/mm/khugepaged.c
+++ b/mm/khugepaged.c
@@ -440,8 +440,12 @@  int khugepaged_enter_vma_merge(struct vm_area_struct *vma,
 		 * page fault if needed.
 		 */
 		return 0;
-	if (vma->vm_ops || (vm_flags & VM_NO_KHUGEPAGED))
-		/* khugepaged not yet working on file or special mappings */
+	if ((vma->vm_ops && !shmem_file(vma->vm_file)) ||
+	    (vm_flags & VM_NO_KHUGEPAGED))
+		/*
+		 * khugepaged not yet working on non-shmem file or special
+		 * mappings
+		 */
 		return 0;
 	hstart = (vma->vm_start + ~HPAGE_PMD_MASK) & HPAGE_PMD_MASK;
 	hend = vma->vm_end & HPAGE_PMD_MASK;
@@ -1517,6 +1521,8 @@  static void collapse_shmem(struct mm_struct *mm,
 		unlock_page(new_page);
 
 		*hpage = NULL;
+
+		khugepaged_pages_collapsed++;
 	} else {
 		/* Something went wrong: rollback changes to the radix-tree */
 		shmem_uncharge(mapping->host, nr_none);